@@ -2413,7 +2413,7 @@ int QsciScintilla::markerDefine(MarkerSymbol sym, int mnr)
     checkMarker(mnr);
 
     if (mnr >= 0)
-        SendScintilla(SCI_MARKERDEFINE, mnr,static_cast<long>(sym));
+        SendScintilla(SCI_MARKERDEFINE, mnr, static_cast<long>(sym));
 
     return mnr;
 }
@@ -2445,7 +2445,7 @@ int QsciScintilla::markerDefine(const QPixmap &pm, int mnr)
 
 
 // Add a marker to a line.
-int QsciScintilla::markerAdd(int linenr,int mnr)
+int QsciScintilla::markerAdd(int linenr, int mnr)
 {
     if (mnr < 0 || mnr > MARKER_MAX || (allocatedMarkers & (1 << mnr)) == 0)
         return -1;
@@ -2590,8 +2590,8 @@ void QsciScintilla::checkMarker(int &mnr)
 {
     if (mnr >= 0)
     {
-        // Check the explicit marker number isn't already allocated.
-        if (mnr > MARKER_MAX || allocatedMarkers & (1 << mnr))
+        // Note that we allow existing markers to be explicitly redefined.
+        if (mnr > MARKER_MAX)
             mnr = -1;
     }
     else
