@@ -2,7 +2,7 @@
 // Scintilla.  It is modelled on QTextEdit - a method of the same name should
 // behave in the same way.
 //
-// Copyright (c) 2008 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -26,11 +26,6 @@
 // http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 // or contact the sales department at sales@riverbankcomputing.com.
 // 
-// This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-// INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-// granted herein.
-// 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
@@ -145,6 +140,9 @@ QsciScintilla::QsciScintilla(QWidget *parent)
 // The dtor.
 QsciScintilla::~QsciScintilla()
 {
+    // Detach any current lexer.
+    detachLexer();
+
     doc.undisplay(this);
     delete stdCmds;
 }
@@ -1233,6 +1231,27 @@ void QsciScintilla::setFolding(FoldStyle folding, int margin)
 }
 
 
+// Clear all current folds.
+void QsciScintilla::clearFolds()
+{
+    recolor();
+
+    int maxLine = SendScintilla(SCI_GETLINECOUNT);
+
+    for (int line = 0; line < maxLine; line++)
+    {
+        int level = SendScintilla(SCI_GETFOLDLEVEL, line);
+
+        if (level & SC_FOLDLEVELHEADERFLAG)
+        {
+            SendScintilla(SCI_SETFOLDEXPANDED, line, 1);
+            foldExpand(line, true, false, 0, level);
+            line--;
+        }
+    }
+}
+
+
 // Set up a folder marker.
 void QsciScintilla::setFoldMarker(int marknr, int mark)
 {
@@ -1718,12 +1737,13 @@ void QsciScintilla::setSelection(int lineFrom, int indexFrom, int lineTo,
 // Set the background colour of selected text.
 void QsciScintilla::setSelectionBackgroundColor(const QColor &col)
 {
-    SendScintilla(SCI_SETSELBACK, 1, col);
-
-    int alpha = qAlpha(col.rgb());
+    int alpha = col.alpha();
     
-    if (alpha < 255)
-        SendScintilla(SCI_SETSELALPHA, alpha);
+    if (alpha == 255)
+        alpha = SC_ALPHA_NOALPHA;
+
+    SendScintilla(SCI_SETSELBACK, 1, col);
+    SendScintilla(SCI_SETSELALPHA, alpha);
 }
 
 
@@ -1780,12 +1800,13 @@ void QsciScintilla::setCaretForegroundColor(const QColor &col)
 // Set the background colour of the line containing the caret.
 void QsciScintilla::setCaretLineBackgroundColor(const QColor &col)
 {
-    SendScintilla(SCI_SETCARETLINEBACK, col);
+    int alpha = col.alpha();
 
-    int alpha = qAlpha(col.rgb());
-    
-    if (alpha < 255)
-        SendScintilla(SCI_SETCARETLINEBACKALPHA, alpha);
+    if (alpha == 255)
+        alpha = SC_ALPHA_NOALPHA;
+
+    SendScintilla(SCI_SETCARETLINEBACK, col);
+    SendScintilla(SCI_SETCARETLINEBACKALPHA, alpha);
 }
 
 
@@ -2394,7 +2415,7 @@ int QsciScintilla::markerDefine(MarkerSymbol sym, int mnr)
     checkMarker(mnr);
 
     if (mnr >= 0)
-        SendScintilla(SCI_MARKERDEFINE, mnr,static_cast<long>(sym));
+        SendScintilla(SCI_MARKERDEFINE, mnr, static_cast<long>(sym));
 
     return mnr;
 }
@@ -2426,7 +2447,7 @@ int QsciScintilla::markerDefine(const QPixmap &pm, int mnr)
 
 
 // Add a marker to a line.
-int QsciScintilla::markerAdd(int linenr,int mnr)
+int QsciScintilla::markerAdd(int linenr, int mnr)
 {
     if (mnr < 0 || mnr > MARKER_MAX || (allocatedMarkers & (1 << mnr)) == 0)
         return -1;
@@ -2511,7 +2532,11 @@ void QsciScintilla::setMarkerBackgroundColor(const QColor &col, int mnr)
 {
     if (mnr <= MARKER_MAX)
     {
-        int alpha = qAlpha(col.rgb());
+        int alpha = col.alpha();
+
+        // An opaque background would make the text invisible.
+        if (alpha == 255)
+            alpha = SC_ALPHA_NOALPHA;
 
         if (mnr < 0)
         {
@@ -2522,9 +2547,7 @@ void QsciScintilla::setMarkerBackgroundColor(const QColor &col, int mnr)
                 if (am & 1)
                 {
                     SendScintilla(SCI_MARKERSETBACK, m, col);
-
-                    if (alpha < 255)
-                        SendScintilla(SCI_MARKERSETALPHA, m, alpha);
+                    SendScintilla(SCI_MARKERSETALPHA, m, alpha);
                 }
 
                 am >>= 1;
@@ -2533,9 +2556,7 @@ void QsciScintilla::setMarkerBackgroundColor(const QColor &col, int mnr)
         else if (allocatedMarkers & (1 << mnr))
         {
             SendScintilla(SCI_MARKERSETBACK, mnr, col);
-
-            if (alpha < 255)
-                SendScintilla(SCI_MARKERSETALPHA, mnr, alpha);
+            SendScintilla(SCI_MARKERSETALPHA, mnr, alpha);
         }
     }
 }
@@ -2559,7 +2580,9 @@ void QsciScintilla::setMarkerForegroundColor(const QColor &col, int mnr)
             }
         }
         else if (allocatedMarkers & (1 << mnr))
+        {
             SendScintilla(SCI_MARKERSETFORE, mnr, col);
+        }
     }
 }
 
@@ -2569,8 +2592,8 @@ void QsciScintilla::checkMarker(int &mnr)
 {
     if (mnr >= 0)
     {
-        // Check the explicit marker number isn't already allocated.
-        if (mnr > MARKER_MAX || allocatedMarkers & (1 << mnr))
+        // Note that we allow existing markers to be explicitly redefined.
+        if (mnr > MARKER_MAX)
             mnr = -1;
     }
     else
@@ -2658,10 +2681,9 @@ void QsciScintilla::setUnmatchedBraceForegroundColor(const QColor &col)
 }
 
 
-// Set the lexer.
-void QsciScintilla::setLexer(QsciLexer *lexer)
+// Detach any lexer.
+void QsciScintilla::detachLexer()
 {
-    // Disconnect any previous lexer.
     if (!lex.isNull())
     {
         lex->setEditor(0);
@@ -2671,6 +2693,14 @@ void QsciScintilla::setLexer(QsciLexer *lexer)
         SendScintilla(SCI_STYLECLEARALL);
         SendScintilla(SCI_CLEARDOCUMENTSTYLE);
     }
+}
+
+
+// Set the lexer.
+void QsciScintilla::setLexer(QsciLexer *lexer)
+{
+    // Detach any current lexer.
+    detachLexer();
 
     // Connect up the new lexer.
     lex = lexer;
